# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from ...commons.types.invoice_id import InvoiceId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class InvoiceUpdate_Set(pydantic.BaseModel):
    type: typing_extensions.Literal["set"]
    value: InvoiceId

    class Config:
        frozen = True
        smart_union = True


class InvoiceUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


InvoiceUpdate = typing.Union[InvoiceUpdate_Set, InvoiceUpdate_Remove]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.date import Date
from .....commons.types.regions import Regions
from .authorized_signatory import AuthorizedSignatory
from .contract_status import ContractStatus
from .insurance_types import InsuranceTypes

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ContractBase(pydantic.BaseModel):
    effective_date: Date = pydantic.Field(description="The starting day upon which the contract is effective")
    expiration_date: typing.Optional[Date] = pydantic.Field(
        default=None, description="An optional end day upon which the contract expires"
    )
    regions: Regions = pydantic.Field(
        description=(
            "The state(s) to which the contract's coverage extends.\n"
            'It may also be set to "national" for the entirety of the US.\n'
        )
    )
    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatory] = None
    commercial_insurance_types: InsuranceTypes = pydantic.Field(
        description=("The commercial plan insurance types this contract applies.\n")
    )
    medicare_insurance_types: InsuranceTypes = pydantic.Field(
        description=("The Medicare plan insurance types this contract applies.\n")
    )
    medicaid_insurance_types: InsuranceTypes = pydantic.Field(
        description=("The Medicaid plan insurance types this contract applies.\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .intake_question import IntakeQuestion
from .patient_history_category_enum import PatientHistoryCategoryEnum

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PatientHistoryCategory(pydantic.BaseModel):
    """
    from candid.resources.encounters.v_4 import (
        IntakeFollowUp,
        IntakeQuestion,
        IntakeResponseAndFollowUps,
        PatientHistoryCategory,
        PatientHistoryCategoryEnum,
    )

    PatientHistoryCategory(
        category=PatientHistoryCategoryEnum.PRESENT_ILLNESS,
        questions=[
            IntakeQuestion(
                id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176",
                text="Do you have any allergies?",
                responses=[
                    IntakeResponseAndFollowUps(
                        response="No allergies",
                        follow_ups=[
                            IntakeFollowUp(
                                id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
                                text="Do you have any allergies?",
                                response="No allergies",
                            )
                        ],
                    )
                ],
            )
        ],
    )
    """

    category: PatientHistoryCategoryEnum
    questions: typing.List[IntakeQuestion] = pydantic.Field(description="Must contain at least one item.")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ....commons.errors.http_service_unavailable_error import HttpServiceUnavailableError
from ....commons.types.http_service_unavailable_error_message import HttpServiceUnavailableErrorMessage

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def submit_eligibility_check(self, *, request: typing.Any) -> typing.Any:
        """
        This API is a wrapper around Change Healthcare's eligibility API. Below are some helpful documentation links:

        - [Change Healthcare - Guides: Contents of the Eligibility Request Body](https://developers.changehealthcare.com/eligibilityandclaims/docs/contents-of-the-eligibility-request-body)
        - [Change Healthcare - Guides: Use "Bare Minimum" Eligibility Requests](https://developers.changehealthcare.com/eligibilityandclaims/docs/use-bare-minimum-eligibility-requests)
        - [Change Healthcare - Guides: Contents of the Eligibility Response](https://developers.changehealthcare.com/eligibilityandclaims/docs/contents-of-the-eligibility-response)
        - [Change Healthcare - Guides: Eligibility JSON-to-EDI API Contents](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-json-to-edi-api-contents)
        - [Change Healthcare - Guides: Eligibility Error Messages](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-error-messages)
        - [Change Healthcare - Guides: FAQ](https://developers.changehealthcare.com/eligibilityandclaims/docs/frequently-asked-questions)
        - [Change Healthcare - Guides: Eligibility FAQs](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-api-requests)
        - [Change Healthcare - Guides: Sandbox API Values and Test Responses](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-sandbox-api-values-and-test-responses)
        - [Change Healthcare - Guides: Sandbox Predefined Fields and Values](https://developers.changehealthcare.com/eligibilityandclaims/docs/sandbox-predefined-fields-and-values)
        - [Change Healthcare - Guides: Using Test Payers in the Sandbox](https://developers.changehealthcare.com/eligibilityandclaims/docs/use-the-test-payers-in-the-sandbox-api)

        A schema of the response object can be found here: [Change Healthcare Docs](https://developers.changehealthcare.com/eligibilityandclaims/reference/medicaleligibility)

        Parameters:
            - request: typing.Any.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/eligibility/v2"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Any, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpServiceUnavailableError":
                raise HttpServiceUnavailableError(
                    pydantic.parse_obj_as(HttpServiceUnavailableErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def submit_eligibility_check(self, *, request: typing.Any) -> typing.Any:
        """
        This API is a wrapper around Change Healthcare's eligibility API. Below are some helpful documentation links:

        - [Change Healthcare - Guides: Contents of the Eligibility Request Body](https://developers.changehealthcare.com/eligibilityandclaims/docs/contents-of-the-eligibility-request-body)
        - [Change Healthcare - Guides: Use "Bare Minimum" Eligibility Requests](https://developers.changehealthcare.com/eligibilityandclaims/docs/use-bare-minimum-eligibility-requests)
        - [Change Healthcare - Guides: Contents of the Eligibility Response](https://developers.changehealthcare.com/eligibilityandclaims/docs/contents-of-the-eligibility-response)
        - [Change Healthcare - Guides: Eligibility JSON-to-EDI API Contents](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-json-to-edi-api-contents)
        - [Change Healthcare - Guides: Eligibility Error Messages](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-error-messages)
        - [Change Healthcare - Guides: FAQ](https://developers.changehealthcare.com/eligibilityandclaims/docs/frequently-asked-questions)
        - [Change Healthcare - Guides: Eligibility FAQs](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-api-requests)
        - [Change Healthcare - Guides: Sandbox API Values and Test Responses](https://developers.changehealthcare.com/eligibilityandclaims/docs/eligibility-sandbox-api-values-and-test-responses)
        - [Change Healthcare - Guides: Sandbox Predefined Fields and Values](https://developers.changehealthcare.com/eligibilityandclaims/docs/sandbox-predefined-fields-and-values)
        - [Change Healthcare - Guides: Using Test Payers in the Sandbox](https://developers.changehealthcare.com/eligibilityandclaims/docs/use-the-test-payers-in-the-sandbox-api)

        A schema of the response object can be found here: [Change Healthcare Docs](https://developers.changehealthcare.com/eligibilityandclaims/reference/medicaleligibility)

        Parameters:
            - request: typing.Any.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/eligibility/v2"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Any, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpServiceUnavailableError":
                raise HttpServiceUnavailableError(
                    pydantic.parse_obj_as(HttpServiceUnavailableErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ContractStatus(str, enum.Enum):
    PENDING = "pending"
    """
    Contract is still in the works and has not been signed and counter-signed yet
    """

    EFFECTIVE = "effective"
    """
    Contract is active and in effect (claims will need to
    be submitted under effective contracts)
    """

    CANCELLED = "cancelled"
    """
    Contract is no longer effective but kept around for historical documentation
    """

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        effective: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ContractStatus.PENDING:
            return pending()
        if self is ContractStatus.EFFECTIVE:
            return effective()
        if self is ContractStatus.CANCELLED:
            return cancelled()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .era_base import EraBase
from .era_id import EraId


class Era(EraBase):
    """
    import uuid

    from candid import Era

    Era(
        era_id=uuid.UUID(
            "4d844ef1-2253-43cd-a4f1-6db7e65cb54b",
        ),
        check_number="CHK12345",
        check_date="2023-10-12",
    )
    """

    era_id: EraId

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .payer_uuid import PayerUuid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Payer(pydantic.BaseModel):
    """
    import uuid

    from candid.resources.payers.v_3 import Payer

    Payer(
        payer_uuid=uuid.UUID(
            "a6431fd2-0712-4714-b1b1-dd094daf9f42",
        ),
        payer_id="12345",
        payer_name="Payer Name",
    )
    """

    payer_uuid: PayerUuid = pydantic.Field(description="Auto-generated ID set on creation.")
    payer_id: str = pydantic.Field(description="The primary national payer ID of the payer.")
    payer_name: str = pydantic.Field(description="The primary display name of the payer.")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .auth_get_token_request import AuthGetTokenRequest
from .auth_get_token_response import AuthGetTokenResponse
from .too_many_requests_error_type import TooManyRequestsErrorType

__all__ = ["AuthGetTokenRequest", "AuthGetTokenResponse", "TooManyRequestsErrorType"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.email import Email
from ...commons.types.phone_number import PhoneNumber
from .patient_base import PatientBase

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PatientCreate(PatientBase):
    phone_numbers: typing.Optional[typing.List[PhoneNumber]] = None
    phone_consent: typing.Optional[bool] = pydantic.Field(default=None, description="Defaults to false")
    email: typing.Optional[Email] = None
    email_consent: typing.Optional[bool] = pydantic.Field(default=None, description="Defaults to false")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....identifiers.types.identifier import Identifier
from ...v_2.types.license_type import LicenseType
from ...v_2.types.organization_provider_address import OrganizationProviderAddress
from ...v_2.types.organization_provider_id import OrganizationProviderId
from ...v_2.types.provider_type import ProviderType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class OrganizationProviderV2(pydantic.BaseModel):
    """
    import datetime
    import uuid

    from candid import (
        Identifier,
        IdentifierCode,
        IdentifierValue_MedicareProviderIdentifier,
        State,
    )
    from candid.resources.organization_providers.v_2 import (
        LicenseType,
        ProviderType,
    )
    from candid.resources.organization_providers.v_3 import OrganizationProviderV2

    OrganizationProviderV2(
        npi="1234567890",
        is_rendering=True,
        is_billing=True,
        first_name="John",
        last_name="Doe",
        organization_name="Acme Medical",
        provider_type=ProviderType.INDIVIDUAL,
        tax_id="123456789",
        taxonomy_code="1234567890",
        license_type=LicenseType.MD,
        employment_start_date=datetime.date.fromisoformat(
            "2020-10-07",
        ),
        employment_termination_date=datetime.date.fromisoformat(
            "2021-10-07",
        ),
        organization_provider_id=uuid.UUID(
            "965a563a-0285-4910-9569-e3739c0f6eab",
        ),
        qualifications=[
            Identifier(
                identifier_id=uuid.UUID(
                    "123e4567-e89b-12d3-a456-426614174000",
                ),
                identifier_code=IdentifierCode.MCR,
                identifier_value=IdentifierValue_MedicareProviderIdentifier(
                    type="medicare_provider_identifier",
                    state=State.CA,
                    provider_number="1234567890",
                ),
            )
        ],
    )
    """

    npi: str = pydantic.Field(
        description="The NPI of the provider. This must be all digits [0-9] and exactly 10 characters long."
    )
    is_rendering: bool = pydantic.Field(description="Whether the provider can be used to render services.")
    is_billing: bool = pydantic.Field(description="Whether the provider can be used to bill services.")
    first_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The first name of the provider, if the provider is an individual."
    )
    last_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The last name of the provider, if the provider is an individual."
    )
    organization_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The name of the provider, if the provider is an organization."
    )
    provider_type: ProviderType = pydantic.Field(
        description=("Whether the provider is an individual (NPPES Type 1) or organization (NPPES Type 2) provider.\n")
    )
    tax_id: typing.Optional[str] = pydantic.Field(
        default=None,
        description=(
            "If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.\n"
        ),
    )
    taxonomy_code: typing.Optional[str] = pydantic.Field(
        default=None, description="A code designating classification and specialization."
    )
    license_type: LicenseType = pydantic.Field(description="The type of license that the provider holds.")
    addresses: typing.Optional[typing.List[OrganizationProviderAddress]] = pydantic.Field(
        default=None, description="The addresses associated with this provider."
    )
    employment_start_date: typing.Optional[dt.date] = pydantic.Field(
        default=None, description="The employment start date for the provider."
    )
    employment_termination_date: typing.Optional[dt.date] = pydantic.Field(
        default=None, description="The employment termination date for the provider."
    )
    organization_provider_id: OrganizationProviderId = pydantic.Field(description="Auto-generated ID set on creation.")
    qualifications: typing.List[Identifier] = pydantic.Field(
        description="Qualification given to a provider (PTAN, Medicaid Provider Id etc.)."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
